@@ -30,7 +30,7 @@ from asn1crypto.keys import (
     PublicKeyInfo,
     RSAPublicKey,
 )
-from pkcs11 import Attribute, KeyType, Mechanism, ObjectClass, Session, Token, lib
+from pkcs11 import Attribute, Key, KeyType, Mechanism, ObjectClass, Session, Token, lib
 from pkcs11.exceptions import (
     GeneralError,
     MultipleObjectsReturned,
@@ -86,6 +86,37 @@ class PKCS11Session:
     lock = Lock()
     session: Session
 
+    @classmethod
+    def _get_pub_key(cls, key_label: str, key_type: str) -> Key:
+        return cls.session.get_key(
+            key_type=key_type_values[key_type],
+            object_class=ObjectClass.PUBLIC_KEY,
+            label=key_label,
+        )
+
+    @classmethod
+    def _public_key_data(cls, key_pub: Key, key_type: str) -> Tuple[str, bytes]:
+        if key_type in ["rsa_2048", "rsa_4096"]:
+            # Create the PublicKeyInfo object
+            rsa_pub = RSAPublicKey.load(encode_rsa_public_key(key_pub))
+
+            pki = PublicKeyInfo()
+            pka = PublicKeyAlgorithm()
+            pka["algorithm"] = PublicKeyAlgorithmId("rsa")
+            pki["algorithm"] = pka
+            pki["public_key"] = rsa_pub
+
+        elif key_type in ["ed25519", "ed448"]:
+            pki = PublicKeyInfo.load(encode_eddsa_public_key(key_pub))
+
+        elif key_type in ["secp256r1", "secp384r1", "secp521r1"]:
+            pki = PublicKeyInfo.load(encode_ec_public_key(key_pub))
+        else:
+            raise ValueError(f"key_type must be in {key_types}")
+
+        key_pub_pem: bytes = asn1_pem.armor("PUBLIC KEY", pki.dump())
+        return key_pub_pem.decode("utf-8"), pki.sha1
+
     @classmethod
     def _open_session(cls, force: Optional[bool] = None, simulate_pkcs11_timeout: Optional[bool] = None) -> None:
         if simulate_pkcs11_timeout:
@@ -179,11 +210,7 @@ class PKCS11Session:
             await cls.healthy_session()
 
             try:
-                key_pub = cls.session.get_key(
-                    key_type=key_type_values[key_type],
-                    object_class=ObjectClass.PUBLIC_KEY,
-                    label=key_label,
-                )
+                key_pub = cls._get_pub_key(key_label, key_type)
                 raise MultipleObjectsReturned
             except NoSuchKey:
                 pass
@@ -238,11 +265,7 @@ class PKCS11Session:
 
             # Try to get the key, if not exist then create it
             try:
-                key_pub = cls.session.get_key(
-                    key_type=key_type_values[key_type],
-                    object_class=ObjectClass.PUBLIC_KEY,
-                    label=key_label,
-                )
+                key_pub = cls._get_pub_key(key_label, key_type)
                 raise MultipleObjectsReturned
             except NoSuchKey:
                 # Generate the rsa keypair
@@ -276,23 +299,7 @@ class PKCS11Session:
                         label=key_label,
                     )
 
-            if key_type in ["rsa_2048", "rsa_4096"]:
-                # Create the PublicKeyInfo object
-                rsa_pub = RSAPublicKey.load(encode_rsa_public_key(key_pub))
-                pki = PublicKeyInfo()
-                pka = PublicKeyAlgorithm()
-                pka["algorithm"] = PublicKeyAlgorithmId("rsa")
-                pki["algorithm"] = pka
-                pki["public_key"] = rsa_pub
-
-            elif key_type in ["ed25519", "ed448"]:
-                pki = PublicKeyInfo.load(encode_eddsa_public_key(key_pub))
-
-            elif key_type in ["secp256r1", "secp384r1", "secp521r1"]:
-                pki = PublicKeyInfo.load(encode_ec_public_key(key_pub))
-
-            key_pub_pem: bytes = asn1_pem.armor("PUBLIC KEY", pki.dump())
-            return key_pub_pem.decode("utf-8"), pki.sha1
+            return cls._public_key_data(key_pub, key_type)
 
     @classmethod
     async def key_labels(cls) -> Dict[str, str]:
@@ -375,11 +382,7 @@ class PKCS11Session:
                 label=key_label,
             )
             if verify_signature:
-                key_pub = cls.session.get_key(
-                    key_type=key_type_values[key_type],
-                    object_class=ObjectClass.PUBLIC_KEY,
-                    label=key_label,
-                )
+                key_pub = cls._get_pub_key(key_label, key_type)
 
             # Sign the data
             signature = key_priv.sign(data, mechanism=mechanism)
@@ -489,11 +492,7 @@ class PKCS11Session:
             await cls.healthy_session()
 
             # Get public key to sign the data with
-            key_pub = cls.session.get_key(
-                key_type=key_type_values[key_type],
-                object_class=ObjectClass.PUBLIC_KEY,
-                label=key_label,
-            )
+            key_pub = cls._get_pub_key(key_label, key_type)
 
             if key_type in ["ed25519", "ed448"]:
                 mech = Mechanism.EDDSA
@@ -586,27 +585,5 @@ class PKCS11Session:
             # Ensure we get a healthy pkcs11 session
             await cls.healthy_session()
 
-            key_pub = cls.session.get_key(
-                key_type=key_type_values[key_type],
-                object_class=ObjectClass.PUBLIC_KEY,
-                label=key_label,
-            )
-
-            if key_type in ["rsa_2048", "rsa_4096"]:
-                # Create the PublicKeyInfo object
-                rsa_pub = RSAPublicKey.load(encode_rsa_public_key(key_pub))
-
-                pki = PublicKeyInfo()
-                pka = PublicKeyAlgorithm()
-                pka["algorithm"] = PublicKeyAlgorithmId("rsa")
-                pki["algorithm"] = pka
-                pki["public_key"] = rsa_pub
-
-            elif key_type in ["ed25519", "ed448"]:
-                pki = PublicKeyInfo.load(encode_eddsa_public_key(key_pub))
-
-            elif key_type in ["secp256r1", "secp384r1", "secp521r1"]:
-                pki = PublicKeyInfo.load(encode_ec_public_key(key_pub))
-
-            key_pub_pem: bytes = asn1_pem.armor("PUBLIC KEY", pki.dump())
-            return key_pub_pem.decode("utf-8"), pki.sha1
+            key_pub = cls._get_pub_key(key_label, key_type)
+            return cls._public_key_data(key_pub, key_type)
